@@ -4,10 +4,13 @@ Started 2015-02-18 by David Megginson
 License: Public Domain
 """
 
+from ast import Try
 import hxl_proxy
 
 import flask, hashlib, hxl, json, logging, pickle, random, re, requests, time, urllib
+from hxl_proxy import caching, exceptions
 
+from urllib.parse import urlparse
 
 # Logger for this module
 logger = logging.getLogger(__name__)
@@ -19,16 +22,23 @@ logger = logging.getLogger(__name__)
 ########################################################################
 
 def make_cache_key (path = None, args_in=None):
-    """Make a key for a caching request, based on the full path.
+    """ Make a key for a caching request, based on the full path.
+
     The cache key depends on the path and the GET parameters,
     excluding &force (so that we can cache the request by using
     the force parameter).
-    @param path: the HTTP path, or None to use the current request
-    @param args_in: the GET params, or None to use the current request
+
+    Args:
+        path(str): the HTTP path, or None to use the current request
+        args_in(dict): the GET params, or None to use the current request
+
+    Returns:
+        str: the cache key string
+
     """
     CACHE_KEY_EXCLUDES = ['force']
 
-    # Fill in default 
+    # Fill in default
     if path is None:
         path = flask.request.path
     if args_in is None:
@@ -46,23 +56,131 @@ def make_cache_key (path = None, args_in=None):
 
 def skip_cache_p ():
     """ Determine whether we are skipping the cache.
-    The HTTP &force parameter requests cache skipping.
-    Note that this function will not detect the &force
-    parameter inside a saved recipe; for now, it has to
-    be specified as a GET param.
-    @returns: True if we don't want to cache
+
+    The HTTP &force parameter requests cache skipping.  Note that this
+    function will not detect the &force parameter inside a saved
+    recipe; for now, it has to be specified as a GET param.
+    
+    Returns:
+        bool: True if we don't want to cache
+
     """
     return True if flask.request.args.get('force') else False
 
+
 
 ########################################################################
-# Input options
+# Input wrappers and options
 ########################################################################
 
+def hxl_data (raw_source, input_options=None):
+    """ Wrapper for hxl.data() to implement a domain-based allow list.
+
+    If the raw_source is a string (i.e. URL), check that the base
+    domain is in the allow list. If the allow list is empty, assume
+    testing and allow any domain.
+
+    Args:
+        raw_source: a HXL data provider, file object, array, or string (representing a URL)
+        input_options (hxl.input.InputOptions): input options for reading a dataset
+
+    Returns:
+        hxl.model.Dataset: a HXL dataset object
+
+    Raises:
+        hxl_proxy.exceptions.DomainNotAllowedError: if the domain for a URL is not in the allow list
+
+    """
+    # don't catch exceptions here; see controllers.py for general exception handling
+    check_allowed_domain(raw_source)
+    return hxl.data(raw_source, input_options)
+
+
+def hxl_make_input (raw_source, input_options=None):
+    """ Wrapper for hxl.make_input() to implement a domain-based allow list.  
+
+    If the raw_source is a string (i.e. URL), check that the base
+    domain is in the allow list. If the allow list is empty, assume
+    testing and allow any domain.
+
+    Args:
+        raw_source: a HXL data provider, file object, array, or string (representing a URL)
+        input_options(hxl.input.InputOptions): input options for reading a dataset
+
+    Returns:
+        hxl.input.AbstractInput: a row-by-row input object (pre-HXL-processing)
+
+    Raises:
+        hxl_proxy.exceptions.DomainNotAllowedError: if the domain for a URL is not in the allow list
+
+    """
+    # don't catch exceptions here; see controllers.py for general exception handling
+    check_allowed_domain(raw_source)
+    return hxl.make_input(raw_source, input_options)
+
+
+def check_allowed_domain (raw_source):
+    """ Raise an exception if raw_source is a URL and its base domain is not in the allow list.
+
+    Args:
+        raw_source: a HXL data source, possibly a string representing a URL
+
+    Raises:
+        hxl_proxy.exceptions.DomainNotAllowedException: if the base domain for a URL is not in the allow list.
+
+    """
+
+    if isinstance(raw_source, str) and not is_allowed_domain(raw_source):
+        logger.error("Domain not in allow list: %s", raw_source)
+        raise exceptions.DomainNotAllowedError(
+            "The HXL Proxy does not allow data from this domain.\n" +
+            "If your data is on HDX, you can use the dataset or resource\n" +
+            "URL with the HXL Proxy. Otherwise, please send requests to\n"
+            "allow new humanitarian-data domains to hdx@un.org.\n\n" +
+            raw_source
+        )
+
+    
+def is_allowed_domain (raw_source):
+    """ Check if raw_source is a URL and its base domain is in the allow list.
+
+    Args:
+        raw_source: a HXL data source, possibly a string representing a URL
+
+    Returns:
+        bool: True if the url is in the allow list, or the list is empty
+
+    """
+    hostnames = hxl_proxy.app.config.get('ALLOWED_DOMAINS_LIST', [])
+
+    # if allowed list is empty just wave in for eveybody. test mode eh?
+    # also, if it's not a string (i.e. URL), proceed
+    if len(hostnames) == 0 or not isinstance(raw_source, str):
+        return True
+
+    url=urlparse(raw_source)
+    # it is a hostname match?
+    if url.netloc in hostnames:
+        return True
+    # maybe an allowed domain child?
+    domains = tuple(['.{}'.format(d) for d in hostnames])
+    if url.netloc.endswith(domains):
+        return True
+    # sorry, call us
+    return False
+
+
 def make_input_options (args):
-    """ Create an InputOptions object from the arguments provided 
+    """ Create an InputOptions object from the parameters provided
+
     Ensure that allow_local is always false. Allow both "-" and "_" between words.
 
+    Args:
+        args(dict): an dictionary of command-line params
+
+    Returns:
+        hxl.input.InputOptions: the input options extracted from the params
+
     """
 
     # set the user agent, to help with analytics
@@ -82,10 +200,17 @@ def make_input_options (args):
             logger.error("Invalid sheet index \"%s\"; defaulting to 0", sheet_index)
             sheet_index = 0
 
+    # get the max timeout from configuration, defaulting to 30 seconds
+    max_timeout = hxl_proxy.app.config.get("MAX_REQUEST_TIMEOUT", 30.0)
+    try:
+        timeout = min(float(args.get("timeout")), max_timeout)
+    except TypeError:
+        timeout = max_timeout
+
     return hxl.input.InputOptions(
         allow_local = False,
         sheet_index = sheet_index,
-        timeout = args.get("timeout", None),
+        timeout = timeout,
         verify_ssl = check_verify_ssl(args),
         http_headers = http_headers,
         selector = args.get("selector", None),
@@ -96,7 +221,15 @@ def make_input_options (args):
 
 
 def check_verify_ssl(args):
-    """Check parameters to see if we need to verify SSL connections."""
+    """ Check parameters to see if we need to verify SSL connections.
+
+    Args:
+        args(dict): command-line parameters
+
+    Returns:
+        bool: True if we need to verify SSL; false if unverified is OK
+
+    """
     if args.get('skip-verify-ssl', args.get("skip_verify_ssl", None)) == 'on':
         return False
     elif args.get('verify_ssl') == 'off' or args.get('verify') == 'off': # deprecated parameters
@@ -110,6 +243,23 @@ def check_verify_ssl(args):
 # Not yet classified
 ########################################################################
 
+def check_markup(s):
+    """ Check if text contains markup or URLs
+    """
+    if re.search("(<[a-zA-Z]|>|href|https?:|\.com|\[url=)", s):
+        return True
+    else:
+        return False
+
+def forbid_markup(s, field_name=None):
+    """ Raise an exception if markup or URLs appear in a string
+    Otherwise, return the string.
+    """
+    if check_markup(s):
+        raise hxl_proxy.exceptions.ForbiddenContentException(s, "markup and URLs not allowed", field_name)
+    else:
+        return s
+
 def clean_tagger_mappings(headers, recipe):
     """ Create a clean list of hashtag mappings for the tagger form
     """
@@ -141,7 +291,7 @@ def clean_tagger_mappings(headers, recipe):
             headers_seen.add(header)
 
     return mappings
-            
+
 
 def make_file_hash(stream):
     """Calculate a hash in chunks from a stream.
@@ -216,7 +366,7 @@ def no_none(s):
     @returns: a string version of the value, or "" if None
     """
     return str(s) if s is not None else ''
-    
+
 def stream_template(template_name, **context):
     """From the flask docs - stream a long template result."""
     hxl_proxy.app.update_template_context(context)
